"""Initial migration

Revision ID: 23
Revises:
Create Date: 2017-12-29 14:39:27.854291

"""

import babelfish
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = '23'
down_revision = '22'


def upgrade():
    pass


def downgrade():
    pass
