# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: search_provider_url_v1.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1csearch_provider_url_v1.proto\x12\x10\x61pp.protobufs.v1\"K\n\x1eSavedSearchProviderUrlResponse\x12\x13\n\x0bprovider_id\x18\x01 \x01(\t\x12\x14\n\x0cprovider_url\x18\x02 \x01(\tb\x06proto3')



_SAVEDSEARCHPROVIDERURLRESPONSE = DESCRIPTOR.message_types_by_name['SavedSearchProviderUrlResponse']
SavedSearchProviderUrlResponse = _reflection.GeneratedProtocolMessageType('SavedSearchProviderUrlResponse', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDSEARCHPROVIDERURLRESPONSE,
  '__module__' : 'search_provider_url_v1_pb2'
  # @@protoc_insertion_point(class_scope:app.protobufs.v1.SavedSearchProviderUrlResponse)
  })
_sym_db.RegisterMessage(SavedSearchProviderUrlResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SAVEDSEARCHPROVIDERURLRESPONSE._serialized_start=50
  _SAVEDSEARCHPROVIDERURLRESPONSE._serialized_end=125
# @@protoc_insertion_point(module_scope)
