# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: announcement_v1.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15\x61nnouncement_v1.proto\x12\x10\x61pp.protobufs.v1\"m\n\x1b\x43reatedAnnouncementResponse\x12\r\n\x05\x61hash\x18\x01 \x01(\t\x12\r\n\x05title\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\r\n\x05image\x18\x04 \x01(\t\x12\x0c\n\x04\x64\x61te\x18\x05 \x01(\t\",\n\x1b\x44\x65letedAnnouncementResponse\x12\r\n\x05\x61hash\x18\x01 \x01(\tb\x06proto3')



_CREATEDANNOUNCEMENTRESPONSE = DESCRIPTOR.message_types_by_name['CreatedAnnouncementResponse']
_DELETEDANNOUNCEMENTRESPONSE = DESCRIPTOR.message_types_by_name['DeletedAnnouncementResponse']
CreatedAnnouncementResponse = _reflection.GeneratedProtocolMessageType('CreatedAnnouncementResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEDANNOUNCEMENTRESPONSE,
  '__module__' : 'announcement_v1_pb2'
  # @@protoc_insertion_point(class_scope:app.protobufs.v1.CreatedAnnouncementResponse)
  })
_sym_db.RegisterMessage(CreatedAnnouncementResponse)

DeletedAnnouncementResponse = _reflection.GeneratedProtocolMessageType('DeletedAnnouncementResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEDANNOUNCEMENTRESPONSE,
  '__module__' : 'announcement_v1_pb2'
  # @@protoc_insertion_point(class_scope:app.protobufs.v1.DeletedAnnouncementResponse)
  })
_sym_db.RegisterMessage(DeletedAnnouncementResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CREATEDANNOUNCEMENTRESPONSE._serialized_start=43
  _CREATEDANNOUNCEMENTRESPONSE._serialized_end=152
  _DELETEDANNOUNCEMENTRESPONSE._serialized_start=154
  _DELETEDANNOUNCEMENTRESPONSE._serialized_end=198
# @@protoc_insertion_point(module_scope)
