# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: updates_v1.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10updates_v1.proto\x12\x10\x61pp.protobufs.v1\"#\n\x12UpdatedAppResponse\x12\r\n\x05\x66orce\x18\x01 \x01(\x08\x62\x06proto3')



_UPDATEDAPPRESPONSE = DESCRIPTOR.message_types_by_name['UpdatedAppResponse']
UpdatedAppResponse = _reflection.GeneratedProtocolMessageType('UpdatedAppResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDAPPRESPONSE,
  '__module__' : 'updates_v1_pb2'
  # @@protoc_insertion_point(class_scope:app.protobufs.v1.UpdatedAppResponse)
  })
_sym_db.RegisterMessage(UpdatedAppResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _UPDATEDAPPRESPONSE._serialized_start=38
  _UPDATEDAPPRESPONSE._serialized_end=73
# @@protoc_insertion_point(module_scope)
