# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: server_certificate_v1.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bserver_certificate_v1.proto\x12\x10\x61pp.protobufs.v1\"J\n\x1eSavedServerCertificateResponse\x12\x13\n\x0b\x63\x65rtificate\x18\x01 \x01(\t\x12\x13\n\x0bprivate_key\x18\x02 \x01(\tb\x06proto3')



_SAVEDSERVERCERTIFICATERESPONSE = DESCRIPTOR.message_types_by_name['SavedServerCertificateResponse']
SavedServerCertificateResponse = _reflection.GeneratedProtocolMessageType('SavedServerCertificateResponse', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDSERVERCERTIFICATERESPONSE,
  '__module__' : 'server_certificate_v1_pb2'
  # @@protoc_insertion_point(class_scope:app.protobufs.v1.SavedServerCertificateResponse)
  })
_sym_db.RegisterMessage(SavedServerCertificateResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SAVEDSERVERCERTIFICATERESPONSE._serialized_start=49
  _SAVEDSERVERCERTIFICATERESPONSE._serialized_end=123
# @@protoc_insertion_point(module_scope)
