"""Initial migration

Revision ID: 6
Revises:
Create Date: 2017-12-29 14:39:27.854291

"""
import babelfish
from alembic import op

# revision identifiers, used by Alembic.

revision = '6'
down_revision = '5'


def upgrade():
    pass


def downgrade():
    pass
